#define F_CPU 16000000
#include <avr/io.h>
#include <util/delay.h>
#include <stdio.h>
#include <stdlib.h>
#include "lcd.h"

#define WIDTH  17
#define HEIGHT 6


struct coord{
    int x;
    int y;
};


struct coord head;
struct coord body[105];
struct coord food;
int size;
int direction; //0=Up, 1=Down, 2=Left, 3=Right
int i,j,k;
int endgame;

//Genereaza coordonatele pentru mancare
struct coord gen_food(void) {
    struct coord c;
	int ok;
    while(1){
	ok=0;
    c.x = (rand() % WIDTH) + 1;
    c.y = (rand() % HEIGHT) + 1;
	for(i = 0; i < size; i++)
		if(body[i].x == c.x && body[i].y == c.y)
			ok = 1;
	if(ok == 0)
		break;
    }
    return c;
}


void init_game(void) {
	size=2;//Initial size
	for(i = 0;i < size; i++) {
		body[i].x = WIDTH/2 - i;
		body[i].y = 2;
	}
	
	head.x = body[0].x;
	head.y = body[0].y;
	
	food = gen_food();
	
}

//Verifica daca exista coliziune/se termina jocul
int collision(void) {
	
for(i = 1;i < size; i++)
	if(body[i].x == head.x && body[i].y == head.y)
		return 1;
return 0;
}

void init_buttons(void) {
	DDRA &= ~(1 << PA0);
	PORTA |= (1 << PA0);
	
	DDRA &= ~(1 << PA1);
	PORTA |= (1 << PA1);
	
	DDRA &= ~(1 << PA2);
	PORTA |= (1 << PD2);
	
	DDRA &= ~(1 << PA3);
	PORTA |= (1 << PA3);	
}

//Verifica daca s-a apasat un buton
void check_input(void) {

	//Up
   if ((PINA & (1 << PA0)) == 0 && direction != 1){
	   direction = 0;
	   _delay_ms(70);
	   }
   //Down
   if ((PINA & (1 << PA1)) == 0 && direction != 0){
	   direction = 1;
	   _delay_ms(70);
	   }
   //Left
   if ((PINA & (1 << PA2)) == 0 && direction != 3){
	   direction = 2;
	   _delay_ms(70);
	   }
   //Right
   if ((PINA & (1 << PA3)) == 0 && direction != 2){
	   direction = 3;
	   _delay_ms(70);
	   }
}

//Modifica pozitia sarpelui
void update(void) {

	switch(direction){
		case 0:
			head.y = (head.y % HEIGHT) + 1;
			break;
		case 1:
			if(head.y == 1)
				head.y = HEIGHT;
			else
				head.y = ((head.y - 1) % HEIGHT);
			break;
		case 2:
			if(head.x == 1)
				head.x = WIDTH;
			else
				head.x = ((head.x - 1) % WIDTH);
			break;
		case 3:
			head.x = (head.x  % WIDTH) + 1;
			break;
	}

	if(collision() == 1)
		endgame = 1;

//Verifica mancare
	if (head.x == food.x && head.y == food.y) {
		size++;
		food = gen_food();
    }
	
//Updateaza pozitie sarpe
	for(i = size - 1; i > 0; i--){
		body[i].x = body[i-1].x;
		body[i].y = body[i-1].y;
	}
	body[0].x=head.x;
	body[0].y=head.y;
	

}

//Afisarea LCD
void draw(void) {
	
	for(i = 0; i < size; i++){
		lcd_goto_xy(body[i].x, body[i].y);
		lcd_chr(BDY);
	}
	
	lcd_goto_xy(food.x, food.y);
	lcd_chr('*');	
}

int main(void){
   init_buttons();    
	endgame=0;
	srand(23);
	lcd_init();
	lcd_contrast(0x40);
	
	//Afisare text/asteptare start
	while(1){
		lcd_goto_xy(7,3);
		lcd_str("SNAKE");
		
		lcd_goto_xy(3,5);
		lcd_str("PRESS BUTTON");
				
		if ((PINA & (1 << PA0)) == 0 || (PINA & (1 << PA1)) == 0|| 
		(PINA & (1 << PA2)) == 0 || (PINA & (1 << PA3)) == 0) {
			break;
		}

		_delay_ms(50);
	
	}
	
	direction=3;
    init_game();
	lcd_clear();
	draw();

    while (1) {
        check_input();
        update();	
		lcd_clear();
		draw();
        _delay_ms(400);
		
		//Verificare daca s-a terminat jocul
		if(endgame==1){
			while(1){
				lcd_clear();
				lcd_goto_xy(5,3);
				if(size==HEIGHT*WIDTH)
					lcd_str("YOU WON");
				else
					lcd_str("YOU LOST");
				
				if ((PINA & (1 << PA0)) == 0 || (PINA & (1 << PA1)) == 0 ||
				 (PINA & (1 << PA2)) == 0 || (PINA & (1 << PA3)) == 0) {
				endgame=0;
				init_game();
				direction=3;
				break;
				}
				_delay_ms(1000);
			}
		}
		
    }
    
    return 0;
}